#include <strings.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/param.h>

#ifdef MIPS
void	*malloc();
void	exit ();
#else
char	*malloc();
#endif MIPS

main (argc, argv)
int	argc;
char	*argv[];
{
        int prev;
	int	low = 0;	/* lowest auto-correlation value */
	int	high;		/* highest auto-correlation value */
	int	count = 100;	/* number of auto-correlation values */
	double	mean;		/* mean of positive values */
	double	sum;		/* sum of products of values */
	double	first;		/* first non-normalized auto-corr value */
	double	total = 0;	/* total of all positive values */
	int	nr = 1000;	/* total number of values */
	int	num = 0;	/* number of positive values */
	int	i, k;
	float  j;
	int	rate = 1;	/* unit difference between successive values */
	char	*ptr;		/* pointer to go through options */
	double	*val;		/* array of values */
	char	line[80];	/* store one input line */


	/* option processing */
	argv++; argc--;

	while (argc > 0) {
	    ptr = *argv;
	    while (*ptr) switch (*ptr++) {

	      case '-':
		break;

	      case 'c':		/* no. of correlation values */
	      case 'C':
		if (*ptr == 0) {
		    argc--; argv++;
		    if (*argv == 0) {
			(void) fprintf (stderr,
"cauto: number of auto-correlation values not given with '-c' option\n");
			exit(1);
		    }
		    count = atoi (*argv);
		}
		else {
		    count = atoi (ptr);
		    *ptr = 0;
		}
		break;

	      case 'h':		/* help */
	      case 'H':
		usage();

	      case 'l':		/* lowest corr value */
	      case 'L':
		if (*ptr == 0) {
		    argc--; argv++;
		    if (*argv == 0) {
			(void) fprintf (stderr,
"cauto: lowest value of auto-correlation not given with '-l' option\n");
			exit(1);
		    }
		    low = atoi (*argv);
		}
		else {
		    low = atoi (ptr);
		    *ptr = 0;
		}
		break;

	      case 'n':		/* number of records */
	      case 'N':
		if (*ptr == 0) {
		    argc--; argv++;
		    if (*argv == 0) {
			(void) fprintf (stderr,
		"cauto: number of records not given with '-n' option\n");
			exit(1);
		    }
		    nr = atoi (*argv);
		}
		else {
		    nr = atoi (ptr);
		    *ptr = 0;
		}
		break;

	      case 'r':		/* rate */
	      case 'R':
		if (*ptr == 0) {
		    argc--; argv++;
		    if (*argv == 0) {
			(void) fprintf (stderr,
		"cauto: rate parameter not given with '-r' option\n");
			exit(1);
		    }
		    rate = atoi (*argv);
		}
		else {
		    rate = atoi (ptr);
		    *ptr = 0;
		}
		break;

	      default:
		(void) fprintf (stderr,
				"cauto: unknown option: '-%c'\n", ptr[-1]);
		exit (1);

	    }
	    argc--; argv++;
	}

	/* alloc space for all values */
	val = (double *) malloc ((unsigned) ((sizeof (double))*(nr+10)));

	high = low + count;		/* highest auto-correlation value */
	i = 0;				/* number of records */

	/* get a new value */
	prev = -999;
	while (gets(line)) {
	    if (!strcmp("",line))
		j = 0;
	    else
	        (void) sscanf (line, "%f", &j);

	    i++;			/* increment no of records */
	    if (j <= 0)
		val[i] = 0;
	    else {
		val[i] = j;		/* val saves all the values */
		total = total + j;
		num++;			/* number of +ve values */
	    }

	    if (i > nr) {
		(void) fprintf (stderr, "number of records > %d, ", nr);
		(void) fprintf (stderr, "ignoring other records\n");
		(void) fprintf (stderr,
			    "you may rerun with higher number of records\n");
		break;
	    }
	}

	nr = i;				/* actual number of records */
	mean = total/num;      	        /* mean of all relevant values */

	for (k=low; k < high; k++) 
	  {
	    double num=0, denom=0;
	    int t;

	    for (t = 1; t <= nr; t++)
	      {
		if ((t+k <= nr) && (val[t] != 0) && (val[t+k] != 0))
		  {
		    num += (val[t]-mean)*(val[t+k]-mean);
		  }
		denom += (val[t]-mean)*(val[t]-mean);
	      }

	    (void) printf ("%d %lf\n", k * rate, num/denom);
	    (void) fflush (stdout);	    
	    
	  }
}

usage()
{
    (void) fprintf (stderr, "usage: cauto <options>\n");
    (void) fprintf (stderr, "The options are: \n");
    (void) fprintf (stderr, "\t -c \t count of auto-correlation values\n");
    (void) fprintf (stderr, "\t -l \t lowest auto-correlation value\n");
    (void) fprintf (stderr, "\t -n \t number of records\n");
    (void) fprintf (stderr, "\t -r \t rate of sending packets\n");
    (void) exit (1);
}

