#include <stdio.h>
#include <stdlib.h>

/* Arguments are 2 files: one with sorted sender sequence numbers, one with
sorted receiver sequence numbers. These files should already be sorted in
ascending order. To create these files from the main data file, execute the 
command:
              cut -f1 -d" " <input file> | sort -n
*/

int
main(argc, argv)
int argc;
char *argv[];
{
  FILE *sendfile, *recvfile;
  int send_gap, recv_gap;
  int curr_s_seq, last_s_seq, curr_r_seq, last_r_seq, recv_count;
  int num_send_losses, num_recv_losses;

  send_gap = recv_gap = 0;

  sendfile = fopen(argv[1],"r");
  recvfile = fopen(argv[2],"r");
  
  if ( (sendfile == NULL) || (recvfile == NULL) )
    {
      printf("USAGE: Invalid filename.\n");
      return 0;
    }

  last_s_seq = last_r_seq = 0;

  while (fscanf(sendfile,"%d",&curr_s_seq) != EOF)
    {
      /* need to compensate specially for duplicate packets since all we
         really want to do is count the number of packets sent (although not
	 all will appear in the data file due to losses). */
      if (curr_s_seq == last_s_seq)
	send_gap++;
      else
	send_gap += curr_s_seq - last_s_seq;
      last_s_seq = curr_s_seq;
    }

  recv_count = 0;
  while (fscanf(recvfile,"%d",&curr_r_seq) != EOF)
    {
      recv_gap += curr_r_seq - last_r_seq;
      last_r_seq = curr_r_seq;
      recv_count++;
    }

/*  printf("send gap %d  recv gap %d  recv count %d\n",send_gap, recv_gap,
	 recv_count);
*/
  num_send_losses = send_gap - recv_gap;
  num_recv_losses = recv_gap - recv_count;

  printf("Number of packets lost on the forward path: %d\n",num_send_losses);
  printf("Number of packets lost on the return path: %d\n",num_recv_losses);

  return 1;
}
