/*
 * tcpecho.c
 * Copyright (c) 1991 University of Maryland
 * All Rights Reserved.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of U.M. not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  U.M. makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * U.M. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL U.M.
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author:  Suman Banerjee, Department of Computer Science.
 * Basically, copied from the udpecho.c
 */

/*
 *	tcpecho.c --- This program runs on the remote host. This receives
 *		messages from the sender program and sends messages to the
 *		receiver program. It adds its own timestamp and the number
 *		of message it has recived so far, to the message.
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <netinet/tcp.h>

#include "udpexpt.h"

#ifdef MIPS
void	*malloc();
void	exit();
void	bzero();
#else
char	*malloc();
#endif MIPS

#define PKTSIZE	32		/* default message size */
#define INDICATOR_SIZE	32

main(argc, argv)
int     argc;
char    *argv[];
{
	struct	sockaddr_in src;	/* Own address/port etc. */
	struct	sockaddr_in dst;	/* Reciver's address/port */
	struct	sockaddr_in fromaddr;	/* sender's address/port */
	int	fromlen;		/* size of sender's address/port */
	int	so1;			/* socket id for incoming pkts */
	int	so2;			/* socket id for outgoing pkts */
	char	data[MAXMESG];		/* maximum size packet */
	int	rcv_size = MAXRCVBUF;	/* socket receive buffer size */
	int	np = -1;		/* sender's packet number */
	int	necho = 0;		/* number of packets recd */
	struct	timeval *tp;		/* local timestamp */
	struct	udprecord *udprecord;	/* log record for the packet */
	char	*ptr;			/* to traverse the arguments */

	/* Extra variables. -- Suman */
	int 	new_so1;		/* Actual socket id for incoming msg */
	int	push = YES;
	int	tcp_no_delay = 1;
	int pktsize = PKTSIZE;
	char indicator[INDICATOR_SIZE];
	int ds_ptr = 0;
	char *data_store;

	/* alloc space for local timestamp */
	tp = (struct timeval *)(malloc (sizeof (struct timeval)));

	argc--; argv++;

	/* go through the arguments */
	while (argc > 0) {
		ptr = *argv;
		while (*ptr) switch (*ptr++) {

		case '-':
			break;

		case 'p':
		case 'P':
         		if (*ptr == 0) {
            		argc--; argv++;
            		if (*argv == 0) {
               			(void) fprintf (stderr,
                               	 "tcpsend: no packet size given with '-p'.\n");
               			(void) exit (1);
            			}
            			pktsize = atoi (*argv);
         		}
         		else {
            			pktsize = atoi (*argv);
            			*ptr = 0;
         		}
			if (MAXMESG < pktsize)
				usage();
         	break;

		default:
			(void) fprintf (stderr,
		"udpecho: Unknown option '%c'\n", ptr[-1]);

		case 'h':		/* help */
		case 'H':
			usage();
		}

		argc--; argv++;
	}

	/* access initial part of packet as log record */
	udprecord = (struct udprecord *) data;

	/* size of socket address data structure for from address */
	fromlen = sizeof (fromaddr);

	/* create a socket for conenction with sender */
	so1 = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (so1 < 0) {
		perror("tcpecho: socket");
		exit(1);
	}

	/* create a socket for connection with receiver */
	so2 = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (so2 < 0) {
		perror("tcpecho: socket");
		exit(1);
	}

	/* initialize address/port for outgoing connection */
	bzero((char *)&dst, sizeof (dst));
	dst.sin_family = AF_INET;
	dst.sin_port = htons(RECVPORT);

	/* initialize address/port etc for incoming connection */
	bzero((char *)&src, sizeof (src));
	src.sin_family = AF_INET;
	src.sin_port = htons(MIDPORT);

	/* bind the address/port number to the socket */
	if (bind (so1, (struct sockaddr *) &src, sizeof(src)) < 0) {
		perror ("udpecho: bind");
		exit (1);
	}

	/* set receive buffers to maximum so that packets are not lost if
	 * for some reason this program does not get CPU for some time. */
	if (setsockopt(so1, SOL_SOCKET, SO_RCVBUF, (char *)&rcv_size,
			sizeof (rcv_size)) < 0) {
		perror ("udpecho: receive buffer option");
		exit (1);
	}

	/* Set the TCP Push flag. -- Suman */
	if (push)
	   if (setsockopt (so2,IPPROTO_TCP,TCP_NODELAY,(char *)&tcp_no_delay,
						sizeof(tcp_no_delay)) < 0) {
		perror ("tcpecho: TCP_NODELAY options");
		exit(1);
	}

	/* First we get a connect from the sender. Then we setup */
	/* a connection to the receiver. -- Suman */
	if (listen (so1,1) < 0) {
		perror("tcpecho: listen");
		exit(1);
	}

	if ((new_so1 = accept (so1, (struct sockaddr *)&fromaddr, &fromlen))
								< 0) {
		perror("tcpecho: accept");
		exit(1);
	}

	/* set the destination address from the 'from' address */
	dst.sin_addr = fromaddr.sin_addr;
	if (connect (so2, (struct sockaddr *)&dst, sizeof(dst)) < 0) {
		perror ("tcpecho: connect");
		exit(1);
	}

	/* Now the echo process needs to tell the sender that all is well */
	/* and it can hence start sending messages. Before that it must get */
	/* the same kind of message from the dest. -- Suman */
	if (read (so2,indicator,INDICATOR_SIZE) < 0)
		{
		 perror("tcpecho: read");
		 exit(1);
		}

	if (write (new_so1,indicator,INDICATOR_SIZE) < 0)
		{
		 perror("tcpecho: write");
		 exit(1);
		}

	ds_ptr = 0;
	data_store = (char *) malloc (sizeof(pktsize));

	/* repeat indefinitely */
	/* TO DO: There should be some clean mechanism to kill the echo
	 * process from remote. Well, may be not. May me it should run as
	 * daemon all the time. Anyway, right now, it can be killed only
	 * by sending a kill signal.
	 */
	while (1) {
		static 	int incc, outcc;
		int data_pos;
		int new_incc;

		/* Before we read in a packet from the sender, we put the */
		/* stuff from data_store in there, so that any partial */
		/* packet that we received till now, will be contiguous */
		if (ds_ptr > 0)
			 memcpy(data,data_store,ds_ptr);
			
		/* receive the packet from the sender */
		new_incc = read (new_so1, (char *)&(data[ds_ptr]), MAXMESG-ds_ptr);

		if (new_incc < 0) {
			perror ("tcpecho: read");
			break;
		}

		incc = new_incc + ds_ptr;

		ds_ptr = 0;
		data_pos = 0;

		/* get the local timestamp */
		(void) gettimeofday (tp, (struct timezone *) 0);

		while ((incc - data_pos) >= pktsize)
			{
			 /* Increment the number of packets received */
			 necho++;

			 udprecord = (struct udprecord *)&(data[data_pos]);

			 /* Add local timestamp and echo number */
			 /* to log record */
			 udprecord->midhost.tv_sec = htonl ((u_long) tp->tv_sec);
			 udprecord->midhost.tv_usec = htonl ((u_long) tp->tv_usec);
			 udprecord->echonum = htonl ((u_long) necho);

			 /* get the sender's packet number */
			 np = ntohl ((u_long) udprecord->num);


			/* the following is in case the write does not */
			/* go thru at one time */
			 /* send it out to the receiver */
			 outcc = 0;
			 do
				{
				 int new_outcc;

				 new_outcc = write (so2, (char *)&(((char*)udprecord)[outcc]), pktsize - outcc);
			 	 if (new_outcc < 0)
					perror ("tcpecho: sendto");
				 outcc += new_outcc;
				}
			 while (outcc < pktsize);

		/*
		 * If the incoming packet has packet-id of 0, then it is to
		 * tell the receiver to stop expecting more packets.
		 * Reset echo number to 0, since the next packets would
		 * be on a new experiment.
		 */
			 if (!np) necho = 0;
			 data_pos += pktsize;
			}

		/* If there is some partial packets that are there */
		/* we have to store this part till the rest appears */
		 if (data_pos != incc)
			{
			 fprintf (stderr,"Pkt %d fragmented and %d bytes was recvd\n",(np+1),new_incc);

			 memcpy(data_store,(char*)&(data[data_pos]),incc-data_pos);
			 ds_ptr = incc - data_pos;
			}
	}

 free(data_store);
 close (so1);
 close (so2);
 close (new_so1);
}


usage()
{
	(void) fprintf (stderr, "usage: tcpecho [-p <packet size>]\n");
	(void) fprintf (stderr,"Keep MAXMESG (%d) >= packet size \n",MAXMESG);
	(void) exit (1);
}

