/*
 * udpread.c
 * Copyright (c) 1991 University of Maryland
 * All Rights Reserved.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of U.M. not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  U.M. makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * U.M. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL U.M.
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author:  Dheeraj Sanghi, Department of Computer Science.
 */

/*
 *	udpread.c --- The reader program. Udprecv sends all the
 *		information regarding packets that are received by it, to
 *		this program. Udpread program writes that information
 *		on a disk file in ASCII format, which can be easily
 *		processed by other tools, such as awk.
 */

#include <stdio.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <netinet/in.h>

#include "udpexpt.h"

#ifdef MIPS
void	exit ();
void	bzero ();
#endif

#define MAXPACKETS	300010		/* max no. of pkts in a connection */
struct packetsStruct {
   int num;			/* packet numbers */
   int echoNum;			/*  echo number to understand loss path */
   double rtt;			/* roundtrip time */
} packets[MAXPACKETS];

int
main(argc, argv)
int	argc;
char	*argv[];
{
   struct	sockaddr_in src;	/* our address/port */
   struct	sockaddr_in dst;	/* remote address/port */
   int	nso, so;		/* socket ids */
   char	*ptr;			/* pointer to go through arguments */
   int	rcv_size = MAXRCVBUF;	/* socket receive buffer size */
   struct	stat	statb;		/* info on output file */
   int	oflag = NO;		/* whether file name given ? */
   int 	rFlag = YES;		/* Write out rtt file(?) (Default = YES) */
   FILE	*fd;			/* output file descriptor */
   char	outfile[MAXFNAME];	/* output file name */
   int	rsize;			/* size of sockaddr_in */
   int	initsecsSnd, initsecsMid;     	/* 1st timestamp - to normalize */
   double	snd1, mid1, rcv1;	/* 3 timestamps in log record */
   struct	udprecord udprecord;	/* log record */
   /* Followin vars are xferred from udprecv ### B Ozgur */
   int	maxnp = 0;		/* highest packet number received */   
   int	total = 0;		/* total number of packets received */
   /* Followin vars are xferred from udprtt ### B Ozgur */
   /* As the rtt calculation is shifted back, we may not need an
      implementation with arrays but to count for lost or out-of-order
      packets, they are kept.   
      (Originally num[] and rtt[] were defined here too (instead of
      being global) but, somehow when the debugger option of cc is
      turned on we get 'Segmentation fault even before starting the
      program(main)
   */
   int	i;		        	/* index variable */
   /* New variables to find duplicates and losses */
   int j, nDuplicates=0, nSendLosses=0, nRetLosses=0, lastEchoNum;
   
   (void) bzero (packets, sizeof(struct packetsStruct) * MAXPACKETS);

   /* argument processing */
   argc--; argv++;

   while (argc > 0) {
      ptr = *argv;
      while (*ptr) switch (*ptr++) {

      case '-':
	 break;

      case 'r':
      case 'R':
	 rFlag = NO;
	 break;
	 
      case 'o':
      case 'O':
	 oflag = YES;
	 if (*ptr == 0) {
	    argc--; argv++;
	    if (*argv == 0) {
	       (void) fprintf (stderr,
			       "udpread: no output file given with '-o'.\n");
	       (void) exit (1);
	    }
	    (void) strcpy (outfile, *argv);
	 }
	 else {
	    (void) strcpy (outfile, ptr);
	    *ptr = 0;
	 }
	 break;

      default:
	 (void) fprintf (stderr,
			 "udpread: Unknown option '%c'\n", ptr[-1]);

      case 'h':
      case 'H':
	 usage();

      }
      argc--; argv++;
   }

   /* If no output file given, ask for it */
   if (!oflag) {
      (void) fprintf (stdout, "Output File? ");
      (void) fflush (stdout);
      (void) gets(outfile);
   }

   /* filename "-" is same as standard output */
   if (!(strcmp (outfile, "-"))) {
      fd = stdout;
      goto nooutfile;
   }

   if (stat (outfile, &statb) < 0) {
      /* If the file does not exist */
      fprintf(stdout, "File doesn't exists!.. Exiting...\n");
      (void) exit (1);
   }
   else
      /* File exists; may be check for directory too */
      fd = fopen(outfile, "r");
   /*
    * To Do: At this time, if the file exists, instead of just
    * opening it in the append mode, may be we should ask the
    * user if he wants to overwrite, or give another file name etc.
    */

   if (fd == NULL) {
      (void) fprintf (stderr, "can't open file %s\n", outfile);
      perror("send: fopen");
      (void) exit (1);
   }

nooutfile:

   /* repeat until connection is broken by the receiver */
   while (!feof(fd)) {
      static int init = YES;		/* is it first packet ? */

      /* read the log record */
      (void) fscanf (fd, "%d %lf %lf %lf %d\n",
		      (int *) &udprecord.num, &snd1, &mid1, &rcv1,
		      (int *) &udprecord.echonum);
      
      /* if this is the first packet */
      if (init) {
	 lastEchoNum = udprecord.echonum;
	 init = NO;
      }

      /* Now we have a normal packet at hand. Do the
         calculations/checks to be written in summary file */
      /* set the maximum packet number received */
      if (maxnp < udprecord.num)
	 maxnp = udprecord.num;
      total++;		/* one more packet received */

      /* Now set the flag for this packet and find its RTT */
      if (packets[udprecord.num].num == 0) {
	 packets[udprecord.num].num = udprecord.num;
	 packets[udprecord.num].echoNum = udprecord.echonum;
	 packets[udprecord.num].rtt = rcv1 - snd1;
      }
      else
	 nDuplicates++;

   }
   (void) fclose (fd);

   /* Now use the same fd to write out rtt file */
   if (rFlag)
      if ( (fd=fopen(strcat(outfile, ".rtt"), "w")) == NULL) {
	 (void) fprintf (stderr, "can't open file %s\n", outfile);
	 perror("send: fopen");
	 (void) exit (1);
      }
   for (i = 1; i <= maxnp; i++)
      /* print the roundtrip times */
      if (packets[i].num > 0) {
	 if (rFlag)
	    (void) fprintf (fd, "%d %6.3lf\n", i, packets[i].rtt);
	 lastEchoNum = packets[i].echoNum;
      }
      else {	/* roundtrip time is reported as -0.001 for lost packets */
	 if (rFlag)
	    (void) fprintf (fd, "%d %6.3lf\n", i, -0.001);
	 for (j=i+1; packets[j].echoNum <= 0; j++); /* Guanteed to terminate */
	 if (packets[j].echoNum - lastEchoNum == 1)
	    nSendLosses++;
	 else {
	    nRetLosses++;
	    lastEchoNum++;
	 }
      }
   
   /* print a bit of stats to stdout -> Summary file */
   (void) fprintf (stdout, "Highest packet no. received = %d\n", maxnp);
   (void) fprintf (stdout, "Number of packets received = %d\n", total);
   (void) fprintf (stdout, "Number of duplicate packets = %d\n", nDuplicates);
   (void) fprintf (stdout, "Number of losses = %d\n", nSendLosses+nRetLosses);
   (void) fprintf (stdout, "Number of losses on send path = %d\n", nSendLosses);
   (void) fprintf (stdout, "Number of losses on receive path = %d\n", nRetLosses);
   if (rFlag)
      (void) fclose (fd);
   
   return 0;
}

usage()
{
   (void) fprintf (stderr,"usage: udpread [-h] [-o outfile]\n");
   (void) exit (1);
}

