/*
 * udprtt.c
 * Copyright (c) 1991 University of Maryland
 * All Rights Reserved.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of U.M. not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  U.M. makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * U.M. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL U.M.
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author:  Dheeraj Sanghi, Department of Computer Science.
 */

/*
 *	udprtt.c --- The program to calculate roundtrip times.
 *		The program looks at the data file of an experiment and
 *		output roundtrip time of all packets. The lost packets
 *		result in a lost time of 0. This could be very misleading
 *		if the roundtrip time is of the order of the clock accuracy.
 */

#include <stdio.h>
#include <strings.h>
#include <sys/time.h>

#include "udpexpt.h"

#define MAXPACKETS	300010		/* max no. of pkts in a connection */

int	num[MAXPACKETS];		/* packet numbers */
double	rtt[MAXPACKETS];		/* roundtrip time */
double	sendtime[MAXPACKETS];		/* send times */

#ifdef MIPS
void	exit ();
#endif

main(argc, argv)
int	argc;
char	*argv[];
{
	int	i, j;		       	/* index variables */
	int	np;			/* packet number */
	int	maxnp;			/* highest packet number */
	int	necho;			/* echo number */
	double	snd1, mid1, rcv1;	/* timestamps */
	double	snd0;			/* 1st send timestamp */
	double	lastsend = 0.0;		/* sendtime of last packet */
	char	line[80];		/* one line of input */
	int	twoway = YES;		/* is it twoway connection */
	int	tflag = NO;		/* rtt vs sendtime ?? */
	char	*ptr;			/* to go through options */
	int	first = YES;		/* is 1st packet */

	argc--; argv++;

	/* go through the options */
	while (argc > 0) {
		ptr = *argv;
		while (*ptr) switch (*ptr++) {

		 case '-':
		   break;

		case '1':
			twoway = NO;	/* one-way connection */
			break;

		case '2':
			twoway = YES;	/* two-way connection */
			break;

		 case 't':
		 case 'T':
		   tflag = YES;		/* sendtime to be calculated as
					 * function of sendtime */
		   break;

		default:
			(void) fprintf (stderr,
		"udprtt: Unknown option '%c'\n", ptr[-1]);

		case 'h':
		case 'H':
			usage();
		}

		argc--; argv++;
	}

	/* initialize packet number array */
	for (i = 0; i < MAXPACKETS; i++)  num[i] = 0;

	maxnp = 0;			/* highet packet number seen is 0 */

	while (gets(line)) {
	   /* get a line of input */
		(void) sscanf (line, "%d %lf %lf %lf %d", &np,
				&snd1, &mid1, &rcv1, &necho);
		if (first) {
		   snd0 = snd1;
		   first = NO;
		}

		if (maxnp < np) maxnp = np;
		if (num[np] == 0) {
		   /* if this is not a duplicate packet */
			num[np] = np;
			if (twoway)	/* if it is two-way */
			   rtt[np] = rcv1 - snd1;
			else		/* if it is one-way */
			   rtt[np] = mid1 - snd1;
			sendtime[np] = snd1 - snd0;
		}
	}

	for (i = 1; i <= maxnp; i++) {
	   /* print the roundtrip times */
		if (num[i] > 0) {
		   if (tflag) {
		      (void) printf ("%6.1lf %6.3lf\n", sendtime[i], rtt[i]);
		      lastsend = sendtime[i];
		   }
		   else
			(void) printf ("%d %6.3lf\n", i, rtt[i]);
		}
		if (num[i] == 0) {
		   /* roundtrip time is reported as -0.001 for lost packets */
		   if (tflag) {
		      for (j = i; j <= maxnp && num[j] <= 0; j++);
		      if (j <= maxnp) {
			 lastsend += (sendtime[j] - lastsend)/(j-i);
			 (void) printf ("%6.1lf %6.3lf\n", lastsend, -0.001);
		      }
		   }
		   else
			(void) printf ("%d %6.3lf\n", i, -0.001);
		}
	}
}

usage()
{
   (void) fprintf (stderr, "usage: udprtt -[1|2] [-t]\n");
   (void) exit (1);
}
